import os
from importlib.metadata import version
from pathlib import Path
from textwrap import dedent


import click
from loguru import logger
import requests


from ytpodgen import downloader, feedgenerator, uploader

SLACK_WEBHOOK_URL = os.environ.get("SLACK_WEBHOOK_URL")

def print_version(ctx, param, value):
    if not value or ctx.resilient_parsing:
        return
    ytpodgen_version = version('ytpodgen')
    click.echo(ytpodgen_version)
    ctx.exit()

def send_slack_notification(message):
    data = {"text": message}
    requests.post(SLACK_WEBHOOK_URL, json=data)

@click.command()
@click.option('--title', prompt='enter title', help='title for the podcast')
@click.option(
    '--hostname',
    prompt='enter hostname',
    help='hostname(custom or r2.dev) to serve files'
)
@click.option(
    '--liveurl',
    help='Watch for and download the specified YouTube Live URL.'
)
@click.option(
    '--upload-r2',
    is_flag=True,
    help='If specified, upload mp3s/RSS to Cloudflare R2.'
)
@click.option(
    '--output',
    help='Output directory(default: current directory)'
)
@click.option(
    '--version',
    is_flag=True,
    callback=print_version,
    expose_value=False,
    is_eager=True,
    help='Show version and exit.'
)

def cli(title, hostname, liveurl, upload_r2, output):
    change_work_dir(output, title)
    create_logger(title)

    try:
        while True:
            run(title, hostname, liveurl, upload_r2)
            if liveurl is None:
                break
    except Exception as e:
        logger.error(f"ytpodgen failed with following error messages: {e}")

def create_logger(title):
    logger.add(f"{title}.log", level="INFO")
    if SLACK_WEBHOOK_URL:
        logger.add(send_slack_notification, level="SUCCESS")

def change_work_dir(output, title):
    try:
        work_dir = Path.cwd()
        if output:
            work_dir = Path(output).absolute()
        os.chdir(work_dir)
    except FileNotFoundError:
        create_logger(title)
        logger.error(f"Invalid argument {output} for --output option.")
        exit()

def run(title, hostname, liveurl, upload_r2):
    if liveurl:
        logger.info("Running yt-dlp...")
        downloader.download(title, liveurl)

    logger.info("Generating feeds...")
    feedgenerator.generate_rss(title, hostname)

    if upload_r2:
        logger.info("Uploading files...")
        uploader.upload_to_r2(title)
        logger.success(dedent(
            f"""
            Upload completed.  Podcast feed url:
            https://{hostname}/{title}/index.rss
            """
            ).strip("\n")
        )

if __name__ == '__main__':
    cli()



import os
from pathlib import Path
import re


from botocore.exceptions import ClientError
import boto3
from loguru import logger


def upload_to_r2(title, bucket='podcast'):
    s3 = boto3.resource(
        's3',
        endpoint_url = get_env_var('R2_ENDPOINT_URL'),
        aws_access_key_id = get_env_var('R2_ACCESS_KEY_ID'),
        aws_secret_access_key = get_env_var('R2_SECRET_ACCESS_KEY'),
    )

    filetypes = '(.+\.mp3|index.rss)' #regex pattern in string
    files = collect_files(filetypes)
    podcastbucket = get_or_create_bucket(bucket, s3)
    upload(title, files, podcastbucket)

def get_env_var(name):
    try:
        envvar = os.environ[name]
        return envvar
    except KeyError:
        raise KeyError(f'{name} not set.')

def get_or_create_bucket(bucket, s3):
    podcastbucket = s3.Bucket(bucket)
    if not bucket_exists(podcastbucket, s3):
        podcastbucket.create()
    return podcastbucket

def collect_files(pattern):
    files = [
        p for p in Path('.').glob('*')
        if re.search(pattern, str(p))
    ]
    if not len(files):
        raise FileNotFoundError('required files not found.')
    return files

def upload(title, upload_files, podcastbucket):
    for file in upload_files:
        try:
            podcastbucket.upload_file(file, f'{title}/{file}')
        except ClientError as e:
            logger.error(e)

def bucket_exists(s3bucket, s3) -> bool:
    return s3bucket in s3.buckets.all()



from pathlib import Path


from podgen import Podcast, Episode, Media


def generate_rss(title, hostname):
    podcast = Podcast(
        name = title,
        website = f'https://{hostname}/{title}/',
        description = title,
        explicit = False,
        withhold_from_itunes = True,
    )
    
    _add_episodes(title, hostname, podcast)
    podcast.rss_file('index.rss')

def _add_episodes(title, hostname, podcast):
    files = Path('.').glob('*.mp3')
    for file in files:
        media = Media(
                url = f'https://{hostname}/{title}/{file.name}',
                size = file.stat().st_size,
            )
        media.populate_duration_from(file)

        podcast.add_episode(Episode(
            title = file.name, #filename
            media = media,
        ))



from yt_dlp import YoutubeDL


from loguru import logger

def download(title, liveurl):
    if _is_live(liveurl):
        live_from_start = True
    else:
        live_from_start = False
    
    # See help(yt_dlp.YoutubeDL) for a list of available options and public functions
    # https://github.com/yt-dlp/yt-dlp/blob/216bcb66d7dce0762767d751dad10650cb57da9d/yt_dlp/YoutubeDL.py#L184
    ydl_opts = {
        'format': 'bestaudio/best', # best audio-only format if available, if not, fall back to best format that contains both video and audio
        'wait_for_video': (60, 60), 
        'outtmpl': f'{title}_%(epoch>%Y%m%d%H%M%S)s_%(id)s',
        'live_from_start': live_from_start,
        'postprocessors': [{  # Extract audio using ffmpeg
            'key': 'FFmpegExtractAudio',
            'preferredcodec': 'mp3',
            'preferredquality': '64'
        }],
        'postprocessor_args': [
            '-ac', '1', # audio channel mono
        ],
    }

    with YoutubeDL(ydl_opts) as ydl:
        error_code = ydl.download(liveurl)

def _is_live(liveurl):
    ydl_opts = {}
    with YoutubeDL(ydl_opts) as ydl:
        try:
            info = ydl.extract_info(liveurl, download=False)
            live_status = info['live_status'].rstrip('\n')
            if live_status == 'is_live':
                return True
        except:
                return False
